/*
 * Copyright 2012 JBoss Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.guvnor.m2repo.backend.server;

import java.io.File;
import java.io.InputStream;
import java.util.Collection;

import org.junit.After;
import org.junit.Ignore;
import org.junit.Test;
import org.guvnor.common.services.project.model.GAV;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.junit.Assert.*;

public class M2RepositoryTest {
    private static final Logger log = LoggerFactory.getLogger( M2RepositoryTest.class );

    
    @After
    public void tearDown() throws Exception {
        log.info( "Creating a new Repository Instance.." );

        File dir = new File( "repository" );
        log.info( "DELETING test repo: " + dir.getAbsolutePath() );
        deleteDir( dir );
        log.info( "TEST repo was deleted." );
    }

    public static boolean deleteDir(File dir) {

        if ( dir.isDirectory() ) {
            String[] children = dir.list();
            for ( int i = 0; i < children.length; i++ ) {
                if ( !deleteDir( new File( dir,
                                           children[i] ) ) ) {
                    return false;
                }
            }
        }

        // The directory is now empty so delete it
        return dir.delete();
    }
    
    @Test
    @Ignore("Fails - ignored for Beta3")
    public void testDeployArtifact() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        Collection<File> files = repo.listFiles();

        boolean found = false;
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("guvnor-m2repo-editor-backend-6.0.0") && fileName.endsWith(".jar")) {
                found = true;
                String path = file.getPath();
                String pom = GuvnorM2Repository.loadPOMFromJar(path);
                assertNotNull(pom);
                break;
            } 
        }
        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found);
    }

    @Test
    public void testListFiles() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        gav = new GAV("org.jboss.arquillian.core", "arquillian-core-api", "1.0.2.Final");        
        is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        Collection<File> files = repo.listFiles();

        boolean found1 = false;
        boolean found2 = false;
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("guvnor-m2repo-editor-backend-6.0.0") && fileName.endsWith(".jar")) {
                found1 = true;
            } 
            if(fileName.startsWith("arquillian-core-api-1.0.2.Final") && fileName.endsWith(".jar")) {
                found2 = true;
            } 
        }
        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found1);
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found2);
    }
    
    @Test
    public void testListFilesWithFilter() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        gav = new GAV("org.jboss.arquillian.core", "arquillian-core-api", "1.0.2.Final");        
        is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        //filter with version number
        Collection<File> files = repo.listFiles("1.0.2");
        boolean found1 = false;
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("arquillian-core-api-1.0.2") && fileName.endsWith(".jar")) {
                found1 = true;
            } 
        }        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found1);
        
/*        //filter with group id
        files = repo.listFiles("org.kie.guvnor");
        found1 = false;
        for(File file : files) {
            String fileName = file.getName();
            if("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar".equals(fileName)) {
                found1 = true;
            } 
        }        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found1);*/
        
        //fileter with artifact id
        files = repo.listFiles("arquillian-core-api");
        found1 = false;
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("arquillian-core-api-1.0.2")  && fileName.endsWith(".jar")) {
                found1 = true;
            } 
        }        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found1);

    }
    
    @Test
    @Ignore("Fails - ignored for Beta3")
    public void testDeleteFile() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        gav = new GAV("org.jboss.arquillian.core", "arquillian-core-api", "1.0.2.Final");        
        is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
        Collection<File> files = repo.listFiles();

        boolean found1 = false;
        boolean found2 = false;
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("guvnor-m2repo-editor-backend-6.0.0") && fileName.endsWith(".jar")) {
                found1 = true;
            } 
            if(fileName.startsWith("arquillian-core-api-1.0.2")  && fileName.endsWith(".jar")) {
                found2 = true;
            } 
        }
        
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found1);
        assertTrue("Did not find expected file after calling M2Repository.addFile()", found2);
        
        boolean result = repo.deleteFile(new String[]{"repository"+ File.separator + "releases"+ File.separator + "org" + File.separator + "kie"+ File.separator + "guvnor"+ File.separator + "guvnor-m2repo-editor-backend"+ File.separator + "6.0.0-SNAPSHOT"+ File.separator + "guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar"});
        result = repo.deleteFile(new String[]{"repository"+ File.separator + "org" + File.separator + "jboss"+ File.separator + "arquillian"+ File.separator + "core"+ File.separator + "arquillian-core-api"+ File.separator + "1.0.2.Final"+ File.separator + "arquillian-core-api-1.0.2.Final.jar"});
        
        found1 = false;
        found2 = false;
        files = repo.listFiles();
        for(File file : files) {
            String fileName = file.getName();
            if(fileName.startsWith("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar") && fileName.endsWith(".jar")) {
                found1 = true;
            } 
            if(fileName.startsWith("arquillian-core-api-1.0.2.Final.jar") && fileName.endsWith(".jar")) {
                found2 = true;
            } 
        }
        
        assertFalse("Found unexpected file after calling M2Repository.deleteFile()", found1);
        assertFalse("Found unexpected file after calling M2Repository.deleteFile()", found2);        
    }
    
    @Test
    public void testLoadPom() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        repo.deployArtifact(is, gav);
        
/*        String pom = repo.loadPOM("repository"+ File.separator + "org"+ File.separator + "kie"+ File.separator + "guvnor"+ File.separator + "guvnor-m2repo-editor-backend"+ File.separator + "6.0.0-SNAPSHOT"+ File.separator + "guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        
        assertNotNull(pom);
        assertTrue(pom.length() > 0);*/
    }
    
    @Test
    public void testLoadPomFromInputStream() throws Exception {
        GuvnorM2Repository repo = new GuvnorM2Repository();
        repo.init();

        GAV gav = new GAV("org.kie.guvnor", "guvnor-m2repo-editor-backend", "6.0.0-SNAPSHOT");
        InputStream is = this.getClass().getResourceAsStream("guvnor-m2repo-editor-backend-6.0.0-SNAPSHOT.jar");
        
/*        String pom = repo.loadPOM(is);
        
        assertNotNull(pom);
        assertTrue(pom.length() > 0);*/
    }
}
